from abc import ABC

from ..interfaces.lint_config import ILintConfig


class MoriLintCase(ABC):
    def __init__(self, lint_config: ILintConfig) -> None:
        self.lint_config = lint_config

    def __call__(self, *args, **kwargs):
        raise NotImplementedError



from dataclasses import dataclass, field


@dataclass(frozen=True)
class LintReportDTO:
    total_checks: int
    success: int
    fail: int
    failed_checks_detail: list[str] = field(default_factory=list)



from dataclasses import dataclass

from ...domain.constants import REQUIRED_KEYWORDS, HEARTS


@dataclass
class LintConfig:
    message_length: int = 34
    req_keywords: list[str] = REQUIRED_KEYWORDS
    hearts: frozenset[str] = HEARTS
    min_hearts_count: int = 1

    check_length: bool = True
    check_keyword: bool = True
    check_hearts: bool = True



def main():
    from .infrastructure.logger.logger import configure_logger
    from .presentation.utils.command_line_parser import CommandLineParser
    from .presentation.api.file_linter.mori_file_linter import MoriFileLinter

    configure_logger()
    directory, config = CommandLineParser.get_config()
    linter = MoriFileLinter(config=config)

    linter.lint(directory=directory)



from dataclasses import dataclass, field


@dataclass(frozen=True)
class LintResponse:
    total_checks: int
    success: int
    fail: int
    failed_checks_detail: list[str] = field(default_factory=list)

    def __str__(self) -> str:
        return (
            f"Проверка завершена {'успешно' if self.fail == 0 else 'неуспешно'}.\n"
            f"Проверок выполнено: {self.total_checks}\n"
            f"Успешно: {self.success}\n"
            f"Ошибки: {self.fail}\n"
            f"Детали по ошибкам: \n-{
                '\n-'.join(self.failed_checks_detail)
            }"
        )



from dataclasses import dataclass

from ..exceptions.base import MoriLintException


@dataclass
class MessageTooShortError(MoriLintException):
    msg_length: int
    permissible_length: int

    @property
    def message(self) -> str:
        return (
                super().message %
                f"Длина сообщения(слов) {self.msg_length} | Минимально-допустимая длина {self.permissible_length}"
        )



from ..exceptions import MessageTooShortError
from ..use_cases.base import MoriLintCase


class ValidateLengthCase(MoriLintCase):
    """
    Сообщение должно быть не менее, чем заданная длина
    """

    def __call__(self, clean_message: str) -> None:
        clean_message_len = len(clean_message.split())
        if clean_message_len < self.lint_config.message_length:
            raise MessageTooShortError(clean_message_len, self.lint_config.message_length)



from dataclasses import FrozenInstanceError

import pytest

from src.morilint.domain.dto import LintReportDTO


class TestLintReportDTO:
    def test_create(self):
        d = LintReportDTO(
            total_checks=5,
            success=3,
            fail=2,
            failed_checks_detail=[
                "Что-то пошло не так..."
            ]
        )
        assert d.total_checks == 5
        assert d.success == 3
        assert d.fail == 2
        assert d.failed_checks_detail == [
                "Что-то пошло не так..."
            ]

    def test_mutable(self):
        d = LintReportDTO(
            total_checks=5,
            success=3,
            fail=2,
            failed_checks_detail=[
                "Что-то пошло не так..."
            ]
        )
        with pytest.raises(FrozenInstanceError):
            d.fail = 10



import pytest


@pytest.fixture
def msg_correct() -> str:
    """
    Полностью корректное сообщение
    """
    return "Корректное сообщение с ❤️ и ключевым словом thewoooooorld, и здесь хватает длины" * 3


@pytest.fixture
def msg_too_short() -> str:
    """
    Сообщение, в котором хватает сердечек
    и есть ключевые слова, но длина меньше заданной
    """
    return "Сообщение с сердечками ❤️ ❤️ и ключевым словом thewoooooorld но не хватает длины"


@pytest.fixture
def msg_no_keywords() -> str:
    """
    Сообщение с правильной длиной, сердечками,
    но нет ключевых слов
    """
    return "Сообщение с правильной длиной, с сердечками ❤️ но без указанных ключевых слов" * 3


@pytest.fixture
def msg_no_hearts() -> str:
    """
    "Сообщение с правильной длиной,
    с указанными ключевыми словами,
    но нет сердечек вообще
    """
    return "Сообщение с правильной длиной, с указанными ключевыми словами thewoooooorld, но вообще без сердечек" * 3


@pytest.fixture
def msg_hearts_not_enough() -> str:
    """
    Сообщение с правильной длиной,
    с указанными ключевыми словами,
    но число сердечек меньше указанного
    """
    return (
            "Сообщение с правильной длиной, с указанными ключевыми словами thewoooooorld, но число сердечек меньше "
            "указанного" * 3 +
            "❤️"
    )


@pytest.fixture
def msg_incorrect() -> str:
    """
    Полностью некорректное сообщение
    """
    return "некорректно :c"



import pytest

from src.morilint.domain.exceptions import MessageHearthMissingError


class TestValidateHeartsIncludeCase:
    def test_call_success(self, validate_hearts_include_case, msg_correct):
        assert validate_hearts_include_case(msg_correct) is None

    def test_call_failed_no_hearths(self, validate_hearts_include_case, msg_no_hearts):
        with pytest.raises(MessageHearthMissingError):
            validate_hearts_include_case(msg_no_hearts)

    def test_call_failed_hearts_not_enough(self, validate_hearts_include_case, msg_hearts_not_enough):
        with pytest.raises(MessageHearthMissingError):
            validate_hearts_include_case(msg_hearts_not_enough)



import pytest

from src.morilint.domain.exceptions import MessageTooShortError


class TestValidateLengthCase:
    def test_call_success(self, validate_length_case, msg_correct):
        assert validate_length_case(msg_correct) is None

    def test_call_message_too_short(self, validate_length_case, msg_too_short):
        with pytest.raises(MessageTooShortError):
            validate_length_case(msg_too_short)




from dataclasses import dataclass

from ..exceptions.base import MoriLintException


@dataclass
class MessageHearthMissingError(MoriLintException):
    hearth_count: int

    @property
    def message(self) -> str:
        return super().message % f"В сообщении должно быть минимум {self.hearth_count} сердечек."



from .base import MoriLintCase
from .validate_length import ValidateLengthCase
from .validate_hearts_include import ValidateHeartsIncludeCase
from .validate_required_keywords import ValidateRequiredKeywordsCase



import argparse

from ...infrastructure.lint_config.config import LintConfig


class CommandLineParser:
    @classmethod
    def read_line_args(cls) -> argparse.Namespace:
        parser = argparse.ArgumentParser(
            description="Чтение и обработка .mori файлов в указанной директории."
        )
        parser.add_argument(
            'directory',
            type=str,
            help="Путь к директории для поиска .mori файлов"
        )

        # Для дополнительных аргументов
        parser.add_argument(
            '--message_length',
            type=int,
            default=79,
            help="Максимальная длина сообщения"
        )
        parser.add_argument(
            '--min_hearts_count',
            type=int, default=1,
            help="Минимальное количество сердечек"
        )

        parser.add_argument(
            '--req_keywords',
            type=lambda s: list(s.split(',')),
            default=list(),
            help="Список обязательных ключевых слов, разделённых запятыми"
        )
        parser.add_argument(
            '--hearts',
            type=lambda s: frozenset(s.split(',')),
            default=frozenset(),
            help="Множество эмодзи сердечек, разделённых запятыми"
        )

        parser.add_argument(
            '--check_length',
            dest='check_length',
            action='store_const',
            const=True,
            default=None,
            help="Включить проверку длины"
        )
        parser.add_argument(
            '--no-check_length',
            dest='check_length',
            action='store_const',
            const=False,
            help="Отключить проверку длины"
        )
        parser.add_argument(
            '--check_keyword',
            dest='check_keyword',
            action='store_const',
            const=True,
            default=None,
            help="Включить проверку ключевых слов"
        )
        parser.add_argument(
            '--no-check_keyword',
            dest='check_keyword',
            action='store_const',
            const=False,
            help="Отключить проверку ключевых слов"
        )
        parser.add_argument(
            '--check_hearts',
            dest='check_hearts',
            action='store_const',
            const=True,
            default=None,
            help="Включить проверку сердечек"
        )
        parser.add_argument(
            '--no-check_hearts',
            dest='check_hearts',
            action='store_const',
            const=False,
            help="Отключить проверку сердечек"
        )

        return parser.parse_args()

    @classmethod
    def get_config(cls) -> tuple[str, LintConfig]:
        config: LintConfig = LintConfig()
        command_line_args: argparse.Namespace = cls.read_line_args()
        if command_line_args.message_length is not None:
            config.message_length = command_line_args.message_length
        if command_line_args.min_hearts_count is not None:
            config.min_hearts_count = command_line_args.min_hearts_count
        if len(command_line_args.req_keywords):
            config.req_keywords = command_line_args.req_keywords
        if len(command_line_args.hearts):
            config.hearts = command_line_args.hearts
        if command_line_args.check_length is not None:
            config.check_length = command_line_args.check_length
        if command_line_args.check_keyword is not None:
            config.check_keyword = command_line_args.check_keyword
        if command_line_args.check_hearts is not None:
            config.check_hearts = command_line_args.check_hearts

        return command_line_args.directory, config



import logging
from pathlib import Path

from rich.table import Table
from rich.console import Console

from ....domain.constants import MORIFILE
from ....infrastructure.lint_config.config import LintConfig
from ....presentation.api.base.mori_linter import MoriLinter
from ....presentation.responses.lint_file_response import LintFilesResponse, LintFileData
from ....presentation.responses.lint_response import LintResponse

logger = logging.getLogger("__mori__")


class MoriFileLinter:
    def __init__(self, config: LintConfig = None) -> None:
        self.__linter = MoriLinter(config=config)
        self.__file_ext = MORIFILE
        self.__console = Console()

    @staticmethod
    def load_file(filepath: Path) -> str:
        with open(filepath, 'r', encoding='utf-8') as f:
            return f.read()

    def lint(self, directory: str, print_lint_result: bool = True) -> LintFilesResponse:
        try:
            files_lint_result: LintFilesResponse = self.lint_files_from_dir(directory)
            if print_lint_result:
                self.print_report_to_console(files_lint_result)
            return files_lint_result
        except Exception as e:
            logger.error(f"Во время проверки файлов произошла неизвестная ошибка: {e}")
            raise e

    def lint_files_from_dir(self, directory: str) -> LintFilesResponse:
        files_lint_result: list[LintFileData] = []
        directory = Path(directory)
        mori_files = directory.glob(self.__file_ext)
        total_files_checked: int = 0
        checked_filenames: list[str] = []
        problems_found: int = 0
        success: int = 0
        failed: int = 0

        for mori_file in mori_files:
            file_res: LintResponse = self.__linter.lint(
                self.load_file(mori_file)
            )
            files_lint_result.append(
                LintFileData(
                    filename=str(mori_file),
                    lint_detail=file_res
                )
            )
            if file_res.fail:
                failed += 1
                problems_found += len(file_res.failed_checks_detail)
            else:
                success += 1
            checked_filenames.append(str(mori_file))
            total_files_checked += 1

        return LintFilesResponse(
            total_files_checked=total_files_checked,
            checked_filenames=checked_filenames,
            problems_found=problems_found,
            success=success,
            failed=failed,
            detail=files_lint_result
        )

    def print_report_to_console(self, file_lint_response: LintFilesResponse) -> None:
        table = Table(title="Mori file lint report")
        table.add_column("Файл", justify="left", style="cyan", no_wrap=True)
        table.add_column("Успешно", style="green")
        table.add_column("Неуспешно", style="red")
        table.add_column("Детали", justify="left", style="bright_red")

        for f_data in file_lint_response.detail:
            table.add_row(
                f_data.filename,
                str(f_data.lint_detail.success),
                str(f_data.lint_detail.fail),
                f"-{
                    '\n-'.join(f_data.lint_detail.failed_checks_detail)
                }\n"
            )

        self.__console.print(table)



import re

from ..exceptions import MessageRequiredKeywordMissingError
from ..use_cases.base import MoriLintCase


class ValidateRequiredKeywordsCase(MoriLintCase):
    """
    В сообщении должно быть хотя-бы одно
    обязательное ключевое слово
    """

    def __call__(self, clean_message: str) -> None:
        pattern = re.compile(
            r'\b(' + '|'.join(map(re.escape, self.lint_config.req_keywords)) + r')\b',
            re.IGNORECASE
        )
        if pattern.search(clean_message) is None:
            raise MessageRequiredKeywordMissingError(self.lint_config.req_keywords)



from typing import Protocol, List, FrozenSet


class ILintConfig(Protocol):
    message_length: int
    req_keywords: List[str]
    hearts: FrozenSet[str]
    min_hearts_count: int
    # для выполнения только определённых проверок
    check_length: bool
    check_keyword: bool
    check_hearts: bool



from ..presentation.responses.lint_file_response import LintFileData, LintFilesResponse
from ..presentation.responses.lint_response import LintResponse
from ..infrastructure.lint_config.config import LintConfig
from ..domain.dto import LintReportDTO
from ..domain.exceptions import (
    MoriLintException,
    MessageTooShortError,
    MessageHearthMissingError,
    MessageRequiredKeywordMissingError
)
from .. import MoriLinter, MoriFileLinter



import logging

from ....domain.dto import LintReportDTO
from ....domain.services.mori_lint_service import MoriLintService
from ....infrastructure.lint_config.config import LintConfig
from ....presentation.responses.lint_response import LintResponse

logger = logging.getLogger("__mori__")


class MoriLinter:
    """
    Интерфейс взаимодействия с сервисом.
    Проверяет одно сообщение и возвращает данные о проверке.
    """
    def __init__(self, config: LintConfig = None) -> None:
        if config is None:
            config = LintConfig()
        self.__service = MoriLintService(lint_config=config)

    def lint(self, message: str) -> LintResponse:
        try:
            report: LintReportDTO = self.__service.lint(message=message)
        except Exception as e:
            logger.error(f"Во время проверки произошла неизвестная ошибка")
            raise e

        return LintResponse(
            total_checks=report.total_checks,
            success=report.success,
            fail=report.fail,
            failed_checks_detail=report.failed_checks_detail
        )



from dataclasses import dataclass

from ..exceptions.base import MoriLintException


@dataclass
class MessageRequiredKeywordMissingError(MoriLintException):
    keywords: list[str]

    @property
    def message(self) -> str:
        return super().message % f"В сообщении НЕ содержатся обязательные слова/выражения: {
            ', '.join(self.keywords)
        }"



from .base import MoriLintException
from .length_exception import MessageTooShortError
from .keyword_exception import MessageRequiredKeywordMissingError
from .hearth_missing_exception import MessageHearthMissingError



import re

from ..exceptions import MessageHearthMissingError
from ..use_cases.base import MoriLintCase


class ValidateHeartsIncludeCase(MoriLintCase):
    """
    В сообщении обязательно должны быть сердечки.
    Не менее, чем в заданной величине.
    """

    def __call__(self, clean_message: str) -> None:
        heart_pattern = re.compile(
            "|".join(re.escape(heart) for heart in self.lint_config.hearts)
        )
        if not self.lint_config.min_hearts_count <= len(heart_pattern.findall(clean_message)):
            raise MessageHearthMissingError(self.lint_config.min_hearts_count)



import logging


def configure_logger(log_level=logging.DEBUG) -> None:
    logger = logging.getLogger("__mori__")
    logger.setLevel(log_level)

    console_handler = logging.StreamHandler()
    console_handler.setLevel(log_level)

    formatter = logging.Formatter("%(asctime)s [%(levelname)s]: %(message)s")
    console_handler.setFormatter(formatter)

    logger.addHandler(console_handler)



import pytest

from src.morilint.domain.exceptions import MessageRequiredKeywordMissingError


class TestValidateRequiredKeywordsCase:
    def test_call_success(self, validate_required_keywords_case, msg_correct):
        assert validate_required_keywords_case(msg_correct) is None

    def test_call_message_required_keyword_missing(self, validate_required_keywords_case, msg_no_keywords):
        with pytest.raises(MessageRequiredKeywordMissingError):
            validate_required_keywords_case(msg_no_keywords)



from dataclasses import dataclass

from ..responses.lint_response import LintResponse


@dataclass(frozen=True)
class LintFileData:
    filename: str
    lint_detail: LintResponse


@dataclass(frozen=True)
class LintFilesResponse:
    total_files_checked: int
    checked_filenames: list[str]
    problems_found: int
    success: int
    failed: int
    detail: list[LintFileData]



import string
from typing import TypeVar, List, Any

from ..dto import LintReportDTO
from ..exceptions import MoriLintException
from ..interfaces.lint_config import ILintConfig
from ..use_cases import (
    ValidateHeartsIncludeCase,
    ValidateLengthCase,
    ValidateRequiredKeywordsCase,
    MoriLintCase
)

T = TypeVar('T', bound=MoriLintCase)


class MoriLintService:
    def __init__(self, lint_config: ILintConfig) -> None:
        self.lint_config: ILintConfig = lint_config

    def lint(self, message: str) -> LintReportDTO:
        """
        Формирует список проверок, на основе конфигурации линтера.
        Передаёт проверки с заданными аргументами на выполнение и
        возвращает результат выполнения проверок.

        :param message: Сообщение от пользователя
        :return: Отчёт с данными о том, сколько всего
            было запущено проверок, сколько успешно/неуспешно, детали не прошедших проверок
        """
        clean_message: str = self.clear_message(message)
        lint_checks_cases: List[List[T | Any]] = []

        if self.lint_config.check_length:
            lint_checks_cases.append([ValidateLengthCase(self.lint_config), clean_message])
        if self.lint_config.check_keyword:
            lint_checks_cases.append([ValidateRequiredKeywordsCase(self.lint_config), clean_message])
        if self.lint_config.check_hearts:
            lint_checks_cases.append([ValidateHeartsIncludeCase(self.lint_config), clean_message])
        return self.__run_and_handle_checks(lint_case_and_args=lint_checks_cases)

    @staticmethod
    def __run_and_handle_checks(lint_case_and_args: List[List[T | Any]]) -> LintReportDTO:
        """
        Вызывает каждую проверку(коллбек) из списка lint_case_and_args с переданными аргументами.
        Обрабатывает возможные исключения, формирует и возвращает данные по проверкам.
        :param lint_case_and_args: Список, где первый элемент - коллбек,
            остальные - произвольное количество элементов для его вызова
        :return: LintReportDTO Отчёт с данными о том, сколько всего было запущено проверок,
            сколько успешно/неуспешно, детали не прошедших проверок
        """

        total = 0
        success = 0
        fail = 0
        failed_checks_detail = []
        for c in lint_case_and_args:
            try:
                total += 1
                c[0](*c[1:])
            except MoriLintException as e:
                fail += 1
                failed_checks_detail.append(e.message)
            else:
                success += 1
        return LintReportDTO(
            total_checks=total,
            success=success,
            fail=fail,
            failed_checks_detail=failed_checks_detail
        )

    @staticmethod
    def clear_message(message: str) -> str:
        """
        Принимает на вход сообщение, убирает из него все знаки препинания,
        лишние пробелы и возвращает очищенную строку.
        :param message: Исходное сообщение
        :return: строка без знаков препинания и лишних пробелов
        """
        return message.translate(
            str.maketrans("", "", string.punctuation)
        ).replace("  ", " ")



from src.morilint.presentation.responses.lint_response import LintResponse


class TestMoriLinter:
    """
    Тест класса MoriLinter - основного публичного класса
    для проверки сообщения
    """
    def test_lint_success(self, linter, msg_correct):
        assert linter.lint(msg_correct) == LintResponse(
            total_checks=3,
            success=3,
            fail=0,
            failed_checks_detail=[]
        )

    def test_lint_failed(self, linter, msg_incorrect):
        assert linter.lint(msg_incorrect) == LintResponse(
            total_checks=3,
            success=0,
            fail=3,
            failed_checks_detail=[
                "Lint Error: [Длина сообщения(слов) 2 | Минимально-допустимая длина 30]",
                "Lint Error: [В сообщении НЕ содержатся обязательные слова/выражения: testword, thewoooooorld]",
                "Lint Error: [В сообщении должно быть минимум 2 сердечек.]",
            ]
        )

    def test_lint_failed_length(self, linter, msg_too_short):
        assert linter.lint(msg_too_short) == LintResponse(
            total_checks=3,
            success=2,
            fail=1,
            failed_checks_detail=[
                "Lint Error: [Длина сообщения(слов) 13 | Минимально-допустимая длина 30]"
            ]
        )

    def test_lint_failed_keywords(self, linter, msg_no_keywords):
        assert linter.lint(msg_no_keywords) == LintResponse(
            total_checks=3,
            success=2,
            fail=1,
            failed_checks_detail=[
                "Lint Error: [В сообщении НЕ содержатся обязательные слова/выражения: testword, thewoooooorld]"
            ]
        )

    def test_lint_failed_no_hearts(self, linter, msg_no_hearts):
        assert linter.lint(msg_no_hearts) == LintResponse(
            total_checks=3,
            success=2,
            fail=1,
            failed_checks_detail=[
                "Lint Error: [В сообщении должно быть минимум 2 сердечек.]"
            ]
        )

    def test_lint_failed_hearts_not_enough(self, linter, msg_hearts_not_enough):
        assert linter.lint(msg_hearts_not_enough) == LintResponse(
            total_checks=3,
            success=2,
            fail=1,
            failed_checks_detail=[
                "Lint Error: [В сообщении должно быть минимум 2 сердечек.]"
            ]
        )



import pytest

from src.tests.msg_fixtures import (
    msg_correct,
    msg_incorrect,
    msg_too_short,
    msg_no_keywords,
    msg_no_hearts,
    msg_hearts_not_enough
)

from src.morilint.domain.use_cases import (
    ValidateHeartsIncludeCase,
    ValidateLengthCase,
    ValidateRequiredKeywordsCase
)


from src.morilint.infrastructure.lint_config.config import LintConfig
from src.morilint.presentation.api.base.mori_linter import MoriLinter


@pytest.fixture
def config() -> LintConfig:
    return LintConfig(
        message_length=30,
        req_keywords=["testword", "thewoooooorld"],
        hearts=frozenset(["❤️", "💖"]),
        min_hearts_count=2,
        check_length=True,
        check_keyword=True,
        check_hearts=True
    )


@pytest.fixture
def linter(config) -> MoriLinter:
    return MoriLinter(config=config)


@pytest.fixture
def validate_hearts_include_case(config) -> ValidateHeartsIncludeCase:
    return ValidateHeartsIncludeCase(config)


@pytest.fixture
def validate_length_case(config) -> ValidateLengthCase:
    return ValidateLengthCase(config)


@pytest.fixture
def validate_required_keywords_case(config) -> ValidateRequiredKeywordsCase:
    return ValidateRequiredKeywordsCase(config)
